@@ -38,6 +38,22 @@ ComputeSizeOfVolumes(DWORD maxvols)
     return size;
 }
 
+afs_uint64
+ComputeSizeOfCellHT(DWORD maxcells)
+{
+    afs_uint64 size;
+    size = osi_PrimeLessThan((afs_uint32)(maxcells/7 + 1)) * sizeof(cm_cell_t *);
+    return size;
+}
+
+afs_uint64
+ComputeSizeOfVolumeHT(DWORD maxvols)
+{
+    afs_uint64 size;
+    size = osi_PrimeLessThan((afs_uint32)(maxvols/7 + 1)) * sizeof(cm_volume_t *);
+    return size;
+}
+
 afs_uint64
 ComputeSizeOfCells(DWORD maxcells)
 {
@@ -66,7 +82,7 @@ afs_uint64
 ComputeSizeOfSCacheHT(DWORD stats)
 {
     afs_uint64 size;
-    size = (stats + 10) / 2 * sizeof(cm_scache_t *);;
+    size = osi_PrimeLessThan(stats / 2 + 1) * sizeof(cm_scache_t *);;
     return size;
 }
 
@@ -109,7 +125,9 @@ ComputeSizeOfMappingFile(DWORD stats, DWORD maxVols, DWORD maxCells, DWORD chunk
     
     size       =  ComputeSizeOfConfigData()
                +  ComputeSizeOfVolumes(maxVols) 
+               +  4 * ComputeSizeOfVolumeHT(maxVols) 
                +  ComputeSizeOfCells(maxCells) 
+               +  2 * ComputeSizeOfCellHT(maxCells)
                +  ComputeSizeOfACLCache(stats)
                +  ComputeSizeOfSCache(stats)
                +  ComputeSizeOfSCacheHT(stats)
@@ -216,18 +234,20 @@ cm_ShutdownMappedMemory(void)
 
     afsi_log("Closing AFS Cache:");
     afsi_log("  Base Address   = %p", config_data_p);
-    afsi_log("  stats          = %d", cm_data.stats);
-    afsi_log("  chunkSize      = %d", cm_data.chunkSize);
-    afsi_log("  blockSize      = %d", cm_data.blockSize);
-    afsi_log("  bufferSize     = %d", cm_data.bufferSize);
-    afsi_log("  cacheType      = %d", cm_data.cacheType);
-    afsi_log("  currentVolumes = %d", cm_data.currentVolumes);
-    afsi_log("  maxVolumes     = %d", cm_data.maxVolumes);
-    afsi_log("  currentCells   = %d", cm_data.currentCells);
-    afsi_log("  maxCells       = %d", cm_data.maxCells);
-    afsi_log("  hashTableSize  = %d", cm_data.hashTableSize);
-    afsi_log("  currentSCaches = %d", cm_data.currentSCaches);
-    afsi_log("  maxSCaches     = %d", cm_data.maxSCaches);
+    afsi_log("  stats          = %u", cm_data.stats);
+    afsi_log("  chunkSize      = %u", cm_data.chunkSize);
+    afsi_log("  blockSize      = %u", cm_data.blockSize);
+    afsi_log("  bufferSize     = %I64u", cm_data.bufferSize);
+    afsi_log("  cacheType      = %u", cm_data.cacheType);
+    afsi_log("  volumeHashTableSize = %u", cm_data.volumeHashTableSize);
+    afsi_log("  currentVolumes = %u", cm_data.currentVolumes);
+    afsi_log("  maxVolumes     = %u", cm_data.maxVolumes);
+    afsi_log("  cellHashTableSize = %u", cm_data.cellHashTableSize);
+    afsi_log("  currentCells   = %u", cm_data.currentCells);
+    afsi_log("  maxCells       = %u", cm_data.maxCells);
+    afsi_log("  scacheHashTableSize  = %u", cm_data.scacheHashTableSize);
+    afsi_log("  currentSCaches = %u", cm_data.currentSCaches);
+    afsi_log("  maxSCaches     = %u", cm_data.maxSCaches);
 
     cm_ShutdownDCache();
     cm_ShutdownSCache();
@@ -399,18 +419,20 @@ cm_ValidateMappedMemory(char * cachePath)
 
     fprintf(stderr,"AFS Cache data:\n");
     fprintf(stderr,"  Base Address   = %p\n",baseAddress);
-    fprintf(stderr,"  stats          = %d\n", config_data_p->stats);
-    fprintf(stderr,"  chunkSize      = %d\n", config_data_p->chunkSize);
-    fprintf(stderr,"  blockSize      = %d\n", config_data_p->blockSize);
-    fprintf(stderr,"  bufferSize     = %d\n", config_data_p->bufferSize);
-    fprintf(stderr,"  cacheType      = %d\n", config_data_p->cacheType);
-    fprintf(stderr,"  currentVolumes = %d\n", config_data_p->currentVolumes);
-    fprintf(stderr,"  maxVolumes     = %d\n", config_data_p->maxVolumes);
-    fprintf(stderr,"  currentCells   = %d\n", config_data_p->currentCells);
-    fprintf(stderr,"  maxCells       = %d\n", config_data_p->maxCells);
-    fprintf(stderr,"  hashTableSize  = %d\n", config_data_p->hashTableSize);
-    fprintf(stderr,"  currentSCaches = %d\n", config_data_p->currentSCaches);
-    fprintf(stderr,"  maxSCaches     = %d\n", config_data_p->maxSCaches);
+    fprintf(stderr,"  stats          = %u\n", config_data_p->stats);
+    fprintf(stderr,"  chunkSize      = %u\n", config_data_p->chunkSize);
+    fprintf(stderr,"  blockSize      = %u\n", config_data_p->blockSize);
+    fprintf(stderr,"  bufferSize     = %I64u\n", config_data_p->bufferSize);
+    fprintf(stderr,"  cacheType      = %u\n", config_data_p->cacheType);
+    fprintf(stderr,"  volumeHashTableSize = %u\n", config_data_p->volumeHashTableSize);
+    fprintf(stderr,"  currentVolumes = %u\n", config_data_p->currentVolumes);
+    fprintf(stderr,"  maxVolumes     = %u\n", config_data_p->maxVolumes);
+    fprintf(stderr,"  cellHashTableSize = %u\n", config_data_p->cellHashTableSize);
+    fprintf(stderr,"  currentCells   = %u\n", config_data_p->currentCells);
+    fprintf(stderr,"  maxCells       = %u\n", config_data_p->maxCells);
+    fprintf(stderr,"  scacheHashTableSize  = %u\n", config_data_p->scacheHashTableSize);
+    fprintf(stderr,"  currentSCaches = %u\n", config_data_p->currentSCaches);
+    fprintf(stderr,"  maxSCaches     = %u\n", config_data_p->maxSCaches);
     cm_data = *config_data_p;      
 
     // perform validation of persisted data structures
@@ -600,14 +622,13 @@ GetMachineSid(PBYTE SidBuffer, DWORD SidSize)
 }
 
 int
-cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chunkSize, afs_uint64 cacheBlocks)
+cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD maxVols, DWORD maxCells,
+                    DWORD chunkSize, afs_uint64 cacheBlocks, afs_uint32 blockSize)
 {
     HANDLE hf = INVALID_HANDLE_VALUE, hm;
     PSECURITY_ATTRIBUTES psa;
     int newFile = 1;
     afs_uint64 mappingSize;
-    DWORD maxVols = stats/2;
-    DWORD maxCells = stats/4;
     DWORD volumeSerialNumber = 0;
     DWORD sidStringSize = 0;
     DWORD rc;
@@ -619,7 +640,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
     volumeSerialNumber = GetVolSerialNumber(cachePath);
     GetMachineSid(machineSid, sizeof(machineSid));
 
-    mappingSize = ComputeSizeOfMappingFile(stats, maxVols, maxCells, chunkSize, cacheBlocks, CM_CONFIGDEFAULT_BLOCKSIZE);
+    mappingSize = ComputeSizeOfMappingFile(stats, maxVols, maxCells, chunkSize, cacheBlocks, blockSize);
 
     if ( !virtualCache ) {
         psa = CreateCacheFileSA();
@@ -734,7 +755,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
                  config_data_p->maxCells == maxCells &&
                  config_data_p->chunkSize == chunkSize &&
                  config_data_p->buf_nbuffers == cacheBlocks &&
-                 config_data_p->blockSize == CM_CONFIGDEFAULT_BLOCKSIZE &&
+                 config_data_p->blockSize == blockSize &&
                  config_data_p->bufferSize == mappingSize)
             {
                 if ( config_data_p->dirty ) {
@@ -797,18 +818,20 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
         cm_data = *config_data_p;      
 
 	afsi_log("  Base Address   = %p",baseAddress);
-	afsi_log("  stats          = %d", config_data_p->stats);
-	afsi_log("  chunkSize      = %d", config_data_p->chunkSize);
-	afsi_log("  blockSize      = %d", config_data_p->blockSize);
-	afsi_log("  bufferSize     = %d", config_data_p->bufferSize);
-	afsi_log("  cacheType      = %d", config_data_p->cacheType);
-	afsi_log("  currentVolumes = %d", config_data_p->currentVolumes);
-	afsi_log("  maxVolumes     = %d", config_data_p->maxVolumes);
-	afsi_log("  currentCells   = %d", config_data_p->currentCells);
-	afsi_log("  maxCells       = %d", config_data_p->maxCells);
-	afsi_log("  hashTableSize  = %d", config_data_p->hashTableSize);
-	afsi_log("  currentSCaches = %d", config_data_p->currentSCaches);
-	afsi_log("  maxSCaches     = %d", config_data_p->maxSCaches);
+	afsi_log("  stats          = %u", config_data_p->stats);
+	afsi_log("  chunkSize      = %u", config_data_p->chunkSize);
+	afsi_log("  blockSize      = %u", config_data_p->blockSize);
+	afsi_log("  bufferSize     = %I64u", config_data_p->bufferSize);
+	afsi_log("  cacheType      = %u", config_data_p->cacheType);
+	afsi_log("  volumeHashTableSize  = %u", config_data_p->volumeHashTableSize);
+	afsi_log("  currentVolumes = %u", config_data_p->currentVolumes);
+	afsi_log("  maxVolumes     = %u", config_data_p->maxVolumes);
+        afsi_log("  cellHashTableSize = %u", config_data_p->cellHashTableSize);
+	afsi_log("  currentCells   = %u", config_data_p->currentCells);
+	afsi_log("  maxCells       = %u", config_data_p->maxCells);
+	afsi_log("  scacheHashTableSize  = %u", config_data_p->scacheHashTableSize);
+	afsi_log("  currentSCaches = %u", config_data_p->currentSCaches);
+	afsi_log("  maxSCaches     = %u", config_data_p->maxSCaches);
 
         // perform validation of persisted data structures
         // if there is a failure, start from scratch
@@ -825,9 +848,11 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
         cm_data.baseAddress = baseAddress;
         cm_data.stats = stats;
         cm_data.chunkSize = chunkSize;
-        cm_data.blockSize = CM_CONFIGDEFAULT_BLOCKSIZE;
+        cm_data.blockSize = blockSize;
         cm_data.bufferSize = mappingSize;
-        cm_data.hashTableSize = osi_PrimeLessThan(stats / 2 + 1);
+        cm_data.scacheHashTableSize = osi_PrimeLessThan(stats / 2 + 1);
+        cm_data.volumeHashTableSize = osi_PrimeLessThan((afs_uint32)(maxVols/7 + 1));
+        cm_data.cellHashTableSize = osi_PrimeLessThan((afs_uint32)(maxCells/7 + 1));
         if (virtualCache) {
             cm_data.cacheType = CM_BUF_CACHETYPE_VIRTUAL;
         } else {
@@ -836,7 +861,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
 
         cm_data.buf_nbuffers = cacheBlocks;
         cm_data.buf_nOrigBuffers = 0;
-        cm_data.buf_blockSize = CM_BUF_BLOCKSIZE;
+        cm_data.buf_blockSize = blockSize;
         cm_data.buf_hashSize = osi_PrimeLessThan((afs_uint32)(cacheBlocks/7 + 1));
 
         cm_data.mountRootGen = time(NULL);
@@ -844,24 +869,36 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
         baseAddress += ComputeSizeOfConfigData();
         cm_data.volumeBaseAddress = (cm_volume_t *) baseAddress;
         baseAddress += ComputeSizeOfVolumes(maxVols);
+        cm_data.volumeNameHashTablep = (cm_volume_t **)baseAddress;
+        baseAddress += ComputeSizeOfVolumeHT(maxVols);
+        cm_data.volumeRWIDHashTablep = (cm_volume_t **)baseAddress;
+        baseAddress += ComputeSizeOfVolumeHT(maxVols);
+        cm_data.volumeROIDHashTablep = (cm_volume_t **)baseAddress;
+        baseAddress += ComputeSizeOfVolumeHT(maxVols);
+        cm_data.volumeBKIDHashTablep = (cm_volume_t **)baseAddress;
+        baseAddress += ComputeSizeOfVolumeHT(maxVols);
+        cm_data.cellNameHashTablep = (cm_cell_t **)baseAddress;
+        baseAddress += ComputeSizeOfCellHT(maxCells);
+        cm_data.cellIDHashTablep = (cm_cell_t **)baseAddress;
+        baseAddress += ComputeSizeOfCellHT(maxCells);
         cm_data.cellBaseAddress = (cm_cell_t *) baseAddress;
         baseAddress += ComputeSizeOfCells(maxCells);
         cm_data.aclBaseAddress = (cm_aclent_t *) baseAddress;
         baseAddress += ComputeSizeOfACLCache(stats);
         cm_data.scacheBaseAddress = (cm_scache_t *) baseAddress;
         baseAddress += ComputeSizeOfSCache(stats);
-        cm_data.hashTablep = (cm_scache_t **) baseAddress;
+        cm_data.scacheHashTablep = (cm_scache_t **) baseAddress;
         baseAddress += ComputeSizeOfSCacheHT(stats);
         cm_data.dnlcBaseAddress = (cm_nc_t *) baseAddress;
         baseAddress += ComputeSizeOfDNLCache();
-        cm_data.buf_hashTablepp = (cm_buf_t **) baseAddress;
+        cm_data.buf_scacheHashTablepp = (cm_buf_t **) baseAddress;
         baseAddress += ComputeSizeOfDataHT(cacheBlocks);
         cm_data.buf_fileHashTablepp = (cm_buf_t **) baseAddress;
         baseAddress += ComputeSizeOfDataHT(cacheBlocks);
         cm_data.bufHeaderBaseAddress = (cm_buf_t *) baseAddress;
         baseAddress += ComputeSizeOfDataHeaders(cacheBlocks);
         cm_data.bufDataBaseAddress = (char *) baseAddress;
-        baseAddress += ComputeSizeOfDataBuffers(cacheBlocks, CM_CONFIGDEFAULT_BLOCKSIZE);
+        baseAddress += ComputeSizeOfDataBuffers(cacheBlocks, blockSize);
         cm_data.bufEndOfData = (char *) baseAddress;
 	cm_data.buf_dirtyListp = NULL;
 	cm_data.buf_dirtyListEndp = NULL;
