@@ -15,8 +15,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_flock.c,v 1.29.2.6 2008/05/23 14:25:16 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -41,105 +39,120 @@ static int lockIdcmp2(struct AFS_FLOCK *flock1, struct vcache *vp,
 static void DoLockWarning(void);
 
 /* int clid;  * non-zero on SGI, OSF, SunOS, Darwin, xBSD ** XXX ptr type */
+
+#if defined(AFS_SUN5_ENV)
 void
 lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
 {
-#if	defined(AFS_SUN5_ENV)
-    register proc_t *procp = ttoproc(curthread);
-#else
-#if !defined(AFS_AIX41_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_SGI65_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_XBSD_ENV)
-#ifdef AFS_SGI_ENV
-    struct proc *procp = OSI_GET_CURRENT_PROCP();
-#else
-    struct proc *procp = u.u_procp;
-#endif /* AFS_SGI_ENV */
-#endif
-#endif
-#if defined(AFS_SGI65_ENV)
-    flid_t flid;
-    get_current_flid(&flid);
-#endif
+    proc_t *procp = ttoproc(curthread);
 
     if (slp) {
-#ifdef	AFS_AIX32_ENV
-#ifdef	AFS_AIX41_ENV
-	slp->sysid = 0;
-	slp->pid = getpid();
-#else
-	slp->sysid = u.u_sysid;
-	slp->pid = u.u_epid;
-#endif
-#else
-#if	defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV)
-#ifdef	AFS_SUN53_ENV
+# ifdef AFS_SUN53_ENV
 	slp->sysid = 0;
 	slp->pid = procp->p_pid;
-#else
+# else
 	slp->sysid = procp->p_sysid;
 	slp->pid = procp->p_epid;
-#endif
-#else
-#if defined(AFS_SGI_ENV)
-#ifdef AFS_SGI65_ENV
-	slp->sysid = flid.fl_sysid;
-#else
-	slp->sysid = OSI_GET_CURRENT_SYSID();
-#endif
-	slp->pid = clid;
-#else
-#if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-	slp->pid = clid;
-#else
-#if defined(AFS_LINUX20_ENV) || defined(AFS_HPUX_ENV)
-	slp->pid = getpid();
-#else
-	slp->pid = u.u_procp->p_pid;
-#endif
-#endif
-#endif /* AFS_AIX_ENV */
-#endif /* AFS_AIX32_ENV */
-#endif
+# endif
     } else {
-#if	defined(AFS_AIX32_ENV)
-#ifdef	AFS_AIX41_ENV
-	flock->l_sysid = 0;
-	flock->l_pid = getpid();
-#else
-	flock->l_sysid = u.u_sysid;
-	flock->l_pid = u.u_epid;
-#endif
-#else
-#if	defined(AFS_AIX_ENV)  || defined(AFS_SUN5_ENV)
-#ifdef	AFS_SUN53_ENV
+# ifdef AFS_SUN53_ENV
 	flock->l_sysid = 0;
 	flock->l_pid = procp->p_pid;
-#else
+# else
 	flock->l_sysid = procp->p_sysid;
 	flock->l_pid = procp->p_epid;
-#endif
-#else
-#if defined(AFS_SGI_ENV)
-#ifdef AFS_SGI65_ENV
-	flock->l_sysid = flid.fl_sysid;
-#else
-	flock->l_sysid = OSI_GET_CURRENT_SYSID();
-#endif
-	flock->l_pid = clid;
-#else
-#if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-	flock->l_pid = clid;
-#else
-#if defined(AFS_LINUX20_ENV) || defined(AFS_HPUX_ENV)
-	flock->l_pid = getpid();
-#else
-	flock->l_pid = u.u_procp->p_pid;
-#endif
-#endif
-#endif
-#endif /* AFS_AIX_ENV */
-#endif /* AFS_AIX32_ENV */
+# endif
     }
 }
+#elif defined(AFS_SGI_ENV)
+void
+lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
+{
+# if defined(AFS_SGI65_ENV)
+    flid_t flid;
+    get_current_flid(&flid);
+# else
+    struct proc *procp = OSI_GET_CURRENT_PROCP();
+# endif
+
+    if (slp) {
+# ifdef AFS_SGI65_ENV
+	slp->sysid = flid.fl_sysid;
+# else
+	slp->sysid = OSI_GET_CURRENT_SYSID();
+# endif
+	slp->pid = clid;
+    } else {
+# ifdef AFS_SGI65_ENV
+	flock->l_sysid = flid.fl_sysid;
+# else
+	flock->l_sysid = OSI_GET_CURRENT_SYSID();
+# endif
+	flock->l_pid = clid;
+    }
+}
+#elif defined(AFS_AIX_ENV)
+void
+lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
+{
+# if !defined(AFS_AIX32_ENV)
+    struct proc *procp = u.u_procp;
+# endif
+
+    if (slp) {
+# if defined(AFS_AIX41_ENV)
+	slp->sysid = 0;
+	slp->pid = getpid();
+# elif defined(AFS_AIX32_ENV)
+	slp->sysid = u.u_sysid;
+	slp->pid = u.u_epid;
+# else
+	slp->sysid = procp->p_sysid;
+	slp->pid = prcop->p_epid;
+# endif
+    } else {
+# if defined(AFS_AIX41_ENV)
+	flock->l_sysid = 0;
+	flock->l_pid = getpid();
+# elif defined(AFS_AIX32_ENV)
+	flock->l_sysid = u.u_sysid;
+	flock->l_pid = u.u_epid;
+# else
+	flock->l_sysid = procp->p_sysid;
+	flock->l_pid = procp->p_epid;
+# endif
+}
+#elif defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+void
+lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
+{
+    if (slp) {
+	slp->pid = clid;
+    } else {
+	flock->l_pid = clid;
+    }
+}
+#elif defined(AFS_LINUX20_ENV) || defined(AFS_HPUX_ENV)
+void
+lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
+{
+    if (slp) {
+	slp->pid = getpid();
+    } else {
+	flock->l_pid = getpid();
+    }
+}
+#else
+void
+lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
+{
+    if (slp) {
+	slp->pid = u.u_procp->p_pid;
+    } else {
+	flock->l_pid = u.u_procp->p_pid;
+    }
+}
+#endif
 
 /* return 1 (true) if specified flock does not match alp (if 
  * specified), or any of the slp structs (if alp == 0) 
@@ -228,7 +241,7 @@ int
 HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 	    pid_t clid, int onlymine)
 {
-    struct conn *tc;
+    struct afs_conn *tc;
     struct SimpleLocks *slp, *tlp, **slpp;
     afs_int32 code;
     struct AFSVolSync tsync;
@@ -300,18 +313,18 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 	if (avc->flockCount == 0) {
 	    if (!AFS_IS_DISCONNECTED) {
 	        do {
-		    tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+		    tc = afs_Conn(&avc->f.fid, areq, SHARED_LOCK);
 		    if (tc) {
 		        XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_RELEASELOCK);
 		        RX_AFS_GUNLOCK();
 		        code = RXAFS_ReleaseLock(tc->id, (struct AFSFid *)
-					         &avc->fid.Fid, &tsync);
+					         &avc->f.fid.Fid, &tsync);
 		        RX_AFS_GLOCK();
 		        XSTATS_END_TIME;
 		    } else
 		    code = -1;
 	        } while (afs_Analyze
-		         (tc, code, &avc->fid, areq,
+		         (tc, code, &avc->f.fid, areq,
 		          AFS_STATS_FS_RPCIDX_RELEASELOCK, SHARED_LOCK, NULL));
 	    } else {
 	  	/*printf("Network is dooooooowwwwwwwnnnnnnn\n");*/
@@ -358,7 +371,7 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 		if (!code && avc->flockCount == 0) {
 		    if (!AFS_IS_DISCONNECTED) {
 		        do {
-			    tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+			    tc = afs_Conn(&avc->f.fid, areq, SHARED_LOCK);
 			    if (tc) {
 			        XSTATS_START_TIME
 				    (AFS_STATS_FS_RPCIDX_RELEASELOCK);
@@ -366,13 +379,13 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 			        code =
 				    RXAFS_ReleaseLock(tc->id,
 						      (struct AFSFid *)&avc->
-						      fid.Fid, &tsync);
+						      f.fid.Fid, &tsync);
 			        RX_AFS_GLOCK();
 			       XSTATS_END_TIME;
 			    } else
 			        code = -1;
 		        } while (afs_Analyze
-			         (tc, code, &avc->fid, areq,
+			         (tc, code, &avc->f.fid, areq,
 			          AFS_STATS_FS_RPCIDX_RELEASELOCK, SHARED_LOCK,
 			          NULL));
 		    }
@@ -393,19 +406,19 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 		    lockType = ((acom & LOCK_EX) ? LockWrite : LockRead);
 		    if (!AFS_IS_DISCONNECTED) {
 		        do {
-			    tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+			    tc = afs_Conn(&avc->f.fid, areq, SHARED_LOCK);
 			    if (tc) {
 			        XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_SETLOCK);
 			        RX_AFS_GUNLOCK();
 			        code = RXAFS_SetLock(tc->id, (struct AFSFid *)
-						     &avc->fid.Fid, lockType,
+						     &avc->f.fid.Fid, lockType,
 						     &tsync);
 			        RX_AFS_GLOCK();
 			        XSTATS_END_TIME;
 			    } else
 			        code = -1;
 		        } while (afs_Analyze
-			         (tc, code, &avc->fid, areq,
+			         (tc, code, &avc->f.fid, areq,
 			          AFS_STATS_FS_RPCIDX_SETLOCK, SHARED_LOCK,
 			          NULL));
 		    } else
@@ -485,8 +498,13 @@ DoLockWarning(void)
 
     /* otherwise, it is time to nag the user */
     lastWarnTime = now;
+#ifdef AFS_LINUX26_ENV
+    afs_warn
+	("afs: byte-range locks only enforced for processes on this machine.\n");
+#else
     afs_warn
 	("afs: byte-range lock/unlock ignored; make sure no one else is running this program.\n");
+#endif
 }
 
 
@@ -570,7 +588,7 @@ int afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
 #endif
     /* Java VMs ask for l_len=(long)-1 regardless of OS/CPU; bottom 32 bits
      * sometimes get masked off by OS */
-    if ((sizeof(af->l_len) == 8) && (af->l_len == 0x7ffffffffffffffe))
+    if ((sizeof(af->l_len) == 8) && (af->l_len == 0x7ffffffffffffffeLL))
 	af->l_len = 0;
     /* next line makes byte range locks always succeed,
      * even when they should block */
@@ -828,7 +846,7 @@ HandleGetLock(register struct vcache *avc, register struct AFS_FLOCK *af,
 static int
 GetFlockCount(struct vcache *avc, struct vrequest *areq)
 {
-    register struct conn *tc;
+    register struct afs_conn *tc;
     register afs_int32 code;
     struct AFSFetchStatus OutStatus;
     struct AFSCallBack CallBack;
@@ -843,19 +861,19 @@ GetFlockCount(struct vcache *avc, struct vrequest *areq)
         return 0;
         
     do {
-	tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+	tc = afs_Conn(&avc->f.fid, areq, SHARED_LOCK);
 	if (tc) {
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_FETCHSTATUS);
 	    RX_AFS_GUNLOCK();
 	    code =
-		RXAFS_FetchStatus(tc->id, (struct AFSFid *)&avc->fid.Fid,
+		RXAFS_FetchStatus(tc->id, (struct AFSFid *)&avc->f.fid.Fid,
 				  &OutStatus, &CallBack, &tsync);
 	    RX_AFS_GLOCK();
 	    XSTATS_END_TIME;
 	} else
 	    code = -1;
     } while (afs_Analyze
-	     (tc, code, &avc->fid, areq, AFS_STATS_FS_RPCIDX_FETCHSTATUS,
+	     (tc, code, &avc->f.fid, areq, AFS_STATS_FS_RPCIDX_FETCHSTATUS,
 	      SHARED_LOCK, NULL));
 
     if (temp)
@@ -982,11 +1000,7 @@ afs_xflock(void)
     return code;
 #else /* AFS_OSF_ENV */
     if (!flockDone)
-#ifdef DYNEL
-	(*afs_longcall_procs.LC_flock) ();
-#else
 	flock();
-#endif
     AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     return;
