@@ -16,8 +16,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_link.c,v 1.15.2.2 2004/10/18 17:43:51 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -32,30 +30,18 @@ extern afs_rwlock_t afs_xcbhash;
  * RPC is called synchronously. */
 
 int
-#ifdef	AFS_OSF_ENV
-afs_link(avc, ndp)
-     struct vcache *avc;
-     struct nameidata *ndp;
-{
-    struct vcache *adp = VTOAFS(ndp->ni_dvp);
-    char *aname = ndp->ni_dent.d_name;
-    struct ucred *acred = ndp->ni_cred;
-#else /* AFS_OSF_ENV */
 #if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
-afs_link(OSI_VC_ARG(adp), avc, aname, acred)
+afs_link(OSI_VC_DECL(adp), struct vcache *avc, char *aname, 
+	 struct AFS_UCRED *acred)
 #else
-afs_link(avc, OSI_VC_ARG(adp), aname, acred)
+afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname, 
+	 struct AFS_UCRED *acred)
 #endif
-     OSI_VC_DECL(adp);
-     struct vcache *avc;
-     char *aname;
-     struct AFS_UCRED *acred;
 {
-#endif
     struct vrequest treq;
     register struct dcache *tdc;
     register afs_int32 code;
-    register struct conn *tc;
+    register struct afs_conn *tc;
     afs_size_t offset, len;
     struct AFSFetchStatus OutFidStatus, OutDirStatus;
     struct AFSVolSync tsync;
@@ -72,6 +58,9 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
 
     afs_InitFakeStat(&vfakestate);
     afs_InitFakeStat(&dfakestate);
+    
+    AFS_DISCON_LOCK();
+
     code = afs_EvalFakeStat(&avc, &vfakestate, &treq);
     if (code)
 	goto done;
@@ -79,8 +68,8 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
     if (code)
 	goto done;
 
-    if (avc->fid.Cell != adp->fid.Cell
-	|| avc->fid.Fid.Volume != adp->fid.Fid.Volume) {
+    if (avc->f.fid.Cell != adp->f.fid.Cell
+	|| avc->f.fid.Fid.Volume != adp->f.fid.Fid.Volume) {
 	code = EXDEV;
 	goto done;
     }
@@ -95,21 +84,26 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
     /** If the volume is read-only, return error without making an RPC to the
       * fileserver
       */
-    if (adp->states & CRO) {
+    if (adp->f.states & CRO) {
 	code = EROFS;
 	goto done;
     }
+    
+    if (AFS_IS_DISCONNECTED) {
+        code = ENETDOWN;
+        goto done;
+    }
 
     tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);	/* test for error below */
     ObtainWriteLock(&adp->lock, 145);
     do {
-	tc = afs_Conn(&adp->fid, &treq, SHARED_LOCK);
+	tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
 	if (tc) {
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_LINK);
 	    RX_AFS_GUNLOCK();
 	    code =
-		RXAFS_Link(tc->id, (struct AFSFid *)&adp->fid.Fid, aname,
-			   (struct AFSFid *)&avc->fid.Fid, &OutFidStatus,
+		RXAFS_Link(tc->id, (struct AFSFid *)&adp->f.fid.Fid, aname,
+			   (struct AFSFid *)&avc->f.fid.Fid, &OutFidStatus,
 			   &OutDirStatus, &tsync);
 	    RX_AFS_GLOCK();
 	    XSTATS_END_TIME;
@@ -117,7 +111,7 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
 	} else
 	    code = -1;
     } while (afs_Analyze
-	     (tc, code, &adp->fid, &treq, AFS_STATS_FS_RPCIDX_LINK,
+	     (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_LINK,
 	      SHARED_LOCK, NULL));
 
     if (code) {
@@ -126,7 +120,7 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
 	if (code < 0) {
 	    ObtainWriteLock(&afs_xcbhash, 492);
 	    afs_DequeueCallback(adp);
-	    adp->states &= ~CStatd;
+	    adp->f.states &= ~CStatd;
 	    ReleaseWriteLock(&afs_xcbhash);
 	    osi_dnlc_purgedp(adp);
 	}
@@ -137,10 +131,12 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
 	ObtainWriteLock(&tdc->lock, 635);
     if (afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
 	/* we can do it locally */
-	code = afs_dir_Create(&tdc->f, aname, &avc->fid.Fid);
+	ObtainWriteLock(&afs_xdcache, 290);
+	code = afs_dir_Create(tdc, aname, &avc->f.fid.Fid);
+	ReleaseWriteLock(&afs_xdcache);
 	if (code) {
 	    ZapDCE(tdc);	/* surprise error -- invalid value */
-	    DZap(&tdc->f);
+	    DZap(tdc);
 	}
     }
     if (tdc) {
@@ -159,9 +155,9 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
 
     ObtainWriteLock(&afs_xcbhash, 493);
     afs_DequeueCallback(avc);
-    avc->states &= ~CStatd;	/* don't really know new link count */
+    avc->f.states &= ~CStatd;	/* don't really know new link count */
     ReleaseWriteLock(&afs_xcbhash);
-    if (avc->fid.Fid.Vnode & 1 || (vType(avc) == VDIR))
+    if (avc->f.fid.Fid.Vnode & 1 || (vType(avc) == VDIR))
 	osi_dnlc_purgedp(avc);
     ReleaseWriteLock(&avc->lock);
     code = 0;
@@ -169,9 +165,7 @@ afs_link(avc, OSI_VC_ARG(adp), aname, acred)
     code = afs_CheckCode(code, &treq, 24);
     afs_PutFakeStat(&vfakestate);
     afs_PutFakeStat(&dfakestate);
+    AFS_DISCON_UNLOCK();
   done2:
-#ifdef	AFS_OSF_ENV
-    afs_PutVCache(adp);
-#endif /* AFS_OSF_ENV */
     return code;
 }
