@@ -23,8 +23,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptutils.c,v 1.19.6.4 2007/10/30 15:16:43 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -42,6 +40,7 @@ RCSID
 #include <afs/cellconfig.h>
 #include "ptserver.h"
 #include "pterror.h"
+#include "ptprototypes.h"
 #include <stdlib.h>
 
 /* Foreign cells are represented by the group system:authuser@cell*/
@@ -51,9 +50,11 @@ extern int restricted;
 extern struct ubik_dbase *dbase;
 extern struct afsconf_dir *prdir;
 extern int pr_noAuth;
-extern int IDCmp();
 
-extern afs_int32 AddToEntry();
+static int inRange(struct prentry *cellEntry, afs_int32 aid);
+static afs_int32 allocNextId(struct ubik_trans *, struct prentry *);
+static int AddAuthGroup(struct prentry *tentry, prlist *alist, afs_int32 *size);
+
 static char *whoami = "ptserver";
 
 int prp_user_default = PRP_USER_DEFAULT;
@@ -64,10 +65,8 @@ int prp_group_default = PRP_GROUP_DEFAULT;
 #include "map.h"
 
 afs_int32 depthsg = 5;		/* Maximum iterations used during IsAMemberOF */
-extern int IDCmp();
 afs_int32 GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist * alist,
 		     afs_int32 * sizeP, afs_int32 depth);
-afs_int32 allocNextId(struct ubik_trans *, struct prentry *);
 
 struct map *sg_flagged;
 struct map *sg_found;
@@ -294,7 +293,7 @@ AccessOK(struct ubik_trans *ut, afs_int32 cid,		/* caller id */
 	return 1;
     if (cid == SYSADMINID)
 	return 1;		/* special case fileserver */
-    if (restricted && ((mem == PRP_ADD_MEM) || (mem == any == 0)))
+    if (restricted && ((mem == PRP_ADD_MEM) || (mem == PRP_REMOVE_MEM)) && (any == 0))
 	return 0;
     if (tentry) {
 	flags = tentry->flags;
@@ -1300,8 +1299,12 @@ GetList(struct ubik_trans *at, struct prentry *tentry, prlist *alist, afs_int32
 		return code;
 #endif
 	}
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 
     if (add) {			/* this is for a CPS, so tack on appropriate stuff */
@@ -1316,8 +1319,10 @@ GetList(struct ubik_trans *at, struct prentry *tentry, prlist *alist, afs_int32
 		return code;
 	}
     }
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
 }
@@ -1376,8 +1381,12 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 #endif
 	}
 	nptr = centry.next;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 
     for (i = 0; i < PRSIZE; i++) {
@@ -1407,8 +1416,12 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 		    break;
 	    }
 	    nptr = centry.next;
-	    if (count++ > 50)
-		IOMGR_Poll(), count = 0;
+	    if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+		IOMGR_Poll();
+#endif
+		count = 0;
+	    }
 	}
     }
     if (add) {			/* this is for a CPS, so tack on appropriate stuff */
@@ -1423,8 +1436,10 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 		return code;
 	}
     }
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
 }
@@ -1529,8 +1544,12 @@ GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist *alist, afs_int32 *sizeP
 		return code;
 	}
 	nptr = centry.next;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 #if DEBUG_SG_MAP
     fprintf(stderr, "] for gid %d, done [flag=%s]\n", gid,
@@ -1589,12 +1608,18 @@ GetSGList(struct ubik_trans *at, struct prentry *tentry, prlist *alist)
 		return code;
 	}
 	nptr = centry.next;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort((char *)alist->prlist_val, (int)alist->prlist_len,
 	  sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
@@ -1623,11 +1648,17 @@ GetOwnedChain(struct ubik_trans *ut, afs_int32 *next, prlist *alist)
 	}
 	if (code)
 	    return code;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
 }
@@ -1681,7 +1712,7 @@ int pr_noAuth;
 afs_int32 initd = 0;
 
 afs_int32
-Initdb()
+Initdb(void)
 {
     afs_int32 code;
     struct ubik_trans *tt;
@@ -2013,7 +2044,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 	tentry.owner = oid;
 	/* The entry must be written through first so Remove and Add routines
 	 * can operate on disk data */
-	code = pr_WriteEntry(at, 0, loc, (char *)&tentry);
+	code = pr_WriteEntry(at, 0, loc, &tentry);
 	if (code)
 	    return PRDBFAIL;
 
@@ -2080,7 +2111,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 	if (code != PRSUCCESS)
 	    return code;
 	strncpy(tentry.name, name, PR_MAXNAMELEN);
-	code = pr_WriteEntry(at, 0, loc, (char *)&tentry);
+	code = pr_WriteEntry(at, 0, loc, &tentry);
 	if (code)
 	    return PRDBFAIL;
 	code = AddToNameHash(at, tentry.name, loc);
@@ -2092,7 +2123,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 }
 
 
-afs_int32
+static afs_int32
 allocNextId(struct ubik_trans * at, struct prentry * cellEntry)
 {
     /* Id's for foreign cell entries are constructed as follows:
@@ -2123,7 +2154,7 @@ allocNextId(struct ubik_trans * at, struct prentry * cellEntry)
     return id;
 }
 
-int
+static int
 inRange(struct prentry *cellEntry, afs_int32 aid)
 {
     afs_uint32 id, cellid, groupid;
@@ -2154,7 +2185,7 @@ inRange(struct prentry *cellEntry, afs_int32 aid)
 
 }
 
-int
+static int
 AddAuthGroup(struct prentry *tentry, prlist *alist, afs_int32 *size)
 {
     if (!(strchr(tentry->name, '@')))
