@@ -12,8 +12,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vol/ntops.c,v 1.10 2004/09/28 05:07:15 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <stdio.h>
@@ -41,6 +39,8 @@ RCSID
 
 #define BASEFILEATTRIBUTE FILE_ATTRIBUTE_NORMAL
 
+int Testing = 0;
+
 static void AddToZLCDeleteList(char dir, char *name);
 
 /* nt_unlink - unlink a case sensitive name.
@@ -520,7 +520,7 @@ nt_MakeSpecIno(int type)
 }
 
 Inode
-nt_icreate(IHandle_t * h, char *part, int p1, int p2, int p3, int p4)
+nt_icreate(IHandle_t * h, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4)
 {
     char filename[128];
     b32_string_t str1;
@@ -837,13 +837,17 @@ nt_GetLCOffsetAndIndexFromIno(Inode ino, int *offset, int *index)
  * If lockit is set, lock the file and leave it locked upon a successful
  * return.
  */
-int
-nt_GetLinkCount(FdHandle_t * h, Inode ino, int lockit)
+static int
+nt_GetLinkCountInternal(FdHandle_t * h, Inode ino, int lockit, int fixup)
 {
     unsigned short row = 0;
-    int junk;
+    DWORD bytesRead, bytesWritten;
     int offset, index;
 
+    /* there's no linktable yet. the salvager will create one later */
+    if (h->fd_fd == INVALID_HANDLE_VALUE && fixup)
+       return 1;
+
     nt_GetLCOffsetAndIndexFromIno(ino, &offset, &index);
 
     if (lockit) {
@@ -854,10 +858,25 @@ nt_GetLinkCount(FdHandle_t * h, Inode ino, int lockit)
     if (!SetFilePointer(h->fd_fd, (LONG) offset, NULL, FILE_BEGIN))
 	goto bad_getLinkByte;
 
-    if (!ReadFile(h->fd_fd, (void *)&row, 2, &junk, NULL)) {
+    if (!ReadFile(h->fd_fd, (void *)&row, 2, &bytesRead, NULL)) 
 	goto bad_getLinkByte;
+    
+    if (bytesRead == 0 && fixup) { 
+	LARGE_INTEGER size;
+
+	if (!GetFileSizeEx(h->fd_fd, &size) || size.QuadPart >= offset+sizeof(row))
+	    goto bad_getLinkByte;
+	FDH_TRUNC(h, offset+sizeof(row));
+	row = 1 << index;
+      rewrite:
+	WriteFile(h->fd_fd, (char *)&row, sizeof(row), &bytesWritten, NULL);
     }
 
+    if (fixup && !((row >> index) & NT_TAGMASK)) {
+        row |= 1<<index;
+        goto rewrite;
+    }
+ 
     return (int)((row >> index) & NT_TAGMASK);
 
   bad_getLinkByte:
@@ -866,7 +885,17 @@ nt_GetLinkCount(FdHandle_t * h, Inode ino, int lockit)
     return -1;
 }
 
+int
+nt_GetLinkCount(FdHandle_t * h, Inode ino, int lockit)
+{
+    return nt_GetLinkCountInternal(h, ino, lockit, 0);
+}
 
+void
+nt_SetNonZLC(FdHandle_t * h, Inode ino) 
+{
+    (void)nt_GetLinkCountInternal(h, ino, 0, 1);
+}
 
 
 /* nt_SetLinkCount
@@ -878,7 +907,7 @@ nt_SetLinkCount(FdHandle_t * h, Inode ino, int count, int locked)
 {
     int offset, index;
     unsigned short row;
-    int junk;
+    DWORD bytesRead, bytesWritten;
     int code = -1;
 
     nt_GetLCOffsetAndIndexFromIno(ino, &offset, &index);
@@ -896,16 +925,16 @@ nt_SetLinkCount(FdHandle_t * h, Inode ino, int count, int locked)
     }
 
 
-    if (!ReadFile(h->fd_fd, (void *)&row, 2, &junk, NULL)) {
+    if (!ReadFile(h->fd_fd, (void *)&row, 2, &bytesRead, NULL)) {
 	errno = nterr_nt2unix(GetLastError(), EBADF);
 	goto bad_SetLinkCount;
     }
-    if (junk == 0)
+    if (bytesRead == 0)
 	row = 0;
 
-    junk = 7 << index;
+    bytesRead = 7 << index;
     count <<= index;
-    row &= (unsigned short)~junk;
+    row &= (unsigned short)~bytesRead;
     row |= (unsigned short)count;
 
     if (!SetFilePointer(h->fd_fd, (LONG) offset, NULL, FILE_BEGIN)) {
@@ -913,7 +942,7 @@ nt_SetLinkCount(FdHandle_t * h, Inode ino, int count, int locked)
 	goto bad_SetLinkCount;
     }
 
-    if (!WriteFile(h->fd_fd, (void *)&row, 2, &junk, NULL)) {
+    if (!WriteFile(h->fd_fd, (void *)&row, 2, &bytesWritten, NULL)) {
 	errno = nterr_nt2unix(GetLastError(), EBADF);
 	goto bad_SetLinkCount;
     }
@@ -930,13 +959,13 @@ nt_SetLinkCount(FdHandle_t * h, Inode ino, int count, int locked)
 
 /* ListViceInodes - write inode data to a results file. */
 static int DecodeInodeName(char *name, int *p1, int *p2);
-static int DecodeVolumeName(char *name, int *vid);
+static int DecodeVolumeName(char *name, afs_uint32 *vid);
 static int nt_ListAFSSubDirs(IHandle_t * dirIH,
 			     int (*write_fun) (FILE *, struct ViceInodeInfo *,
 					       char *, char *), FILE * fp,
 			     int (*judgeFun) (struct ViceInodeInfo *,
-					      int vid, void *rock),
-			     int singleVolumeNumber, void *rock);
+					      afs_uint32 vid, void *rock),
+			     afs_uint32 singleVolumeNumber, void *rock);
 
 
 /* WriteInodeInfo
@@ -972,8 +1001,8 @@ WriteInodeInfo(FILE * fp, struct ViceInodeInfo *info, char *dir, char *name)
  */
 int
 ListViceInodes(char *devname, char *mountedOn, char *resultFile,
-	       int (*judgeInode) (struct ViceInodeInfo * info, int vid, void *rock),
-	       int singleVolumeNumber, int *forcep, int forceR, char *wpath, 
+	       int (*judgeInode) (struct ViceInodeInfo * info, afs_uint32 vid, void *rock),
+	       afs_uint32 singleVolumeNumber, int *forcep, int forceR, char *wpath, 
 	       void *rock)
 {
     FILE *fp = (FILE *) - 1;
@@ -1042,8 +1071,8 @@ int
 nt_ListAFSFiles(char *dev,
 		int (*writeFun) (FILE *, struct ViceInodeInfo *, char *,
 				 char *), FILE * fp,
-		int (*judgeFun) (struct ViceInodeInfo *, int, void *),
-		int singleVolumeNumber, void *rock)
+		int (*judgeFun) (struct ViceInodeInfo *, afs_uint32, void *),
+		afs_uint32 singleVolumeNumber, void *rock)
 {
     IHandle_t h;
     char name[MAX_PATH];
@@ -1095,8 +1124,8 @@ static int
 nt_ListAFSSubDirs(IHandle_t * dirIH,
 		  int (*writeFun) (FILE *, struct ViceInodeInfo *, char *,
 				   char *), FILE * fp,
-		  int (*judgeFun) (struct ViceInodeInfo *, int, void *),
-		  int singleVolumeNumber, void *rock)
+		  int (*judgeFun) (struct ViceInodeInfo *, afs_uint32, void *),
+		  afs_uint32 singleVolumeNumber, void *rock)
 {
     int i;
     IHandle_t myIH = *dirIH;
@@ -1208,7 +1237,7 @@ nt_ListAFSSubDirs(IHandle_t * dirIH,
 
 /* The name begins with "Vol_" and ends with .data.  See nt_HandleToVolDir() */
 static int
-DecodeVolumeName(char *name, int *vid)
+DecodeVolumeName(char *name, afs_uint32 *vid)
 {
     char stmp[32];
     int len;
